#!/usr/bin/env python
#
# Copyright (c) Members of the EGEE Collaboration. 2009-2010.
# See http://www.eu-egee.org/partners/ for details on the copyright
# holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import pexpect, sys

def fail(msg):
    print >> sys.stderr, msg
    sys.exit(1)

def checkStatus(child, val):
    child.expect(pexpect.EOF)
    child.close()
    stat = child.exitstatus
    if stat != val: fail("Status was " + `stat` + " rather than " + `val`)

    

child = pexpect.spawn('saga-isn -h')
child.expect("Usage:")
checkStatus(child, 0)

child = pexpect.spawn('saga-isn -w')
child.expect("Unexpected switch '-w'")
checkStatus(child, 1)

child = pexpect.spawn("saga-isn glue1 Service")
lines = child.readlines()
if len(lines) != 58375: fail("Wrong number of lines returned")
checkStatus(child, 0)

child = pexpect.spawn("saga-isn", ['-n', "1", "glue1", 'Service', "[Name = 'glite-data-transfer-fts_fts.esac.esa.int-channel']"])
child.expect("Owner : dteam")
child.expect("Owner : ops")
child.expect("Owner : planck")
checkStatus(child, 0)

child = pexpect.spawn("saga-isn", ["-t", "glue1", "ComputingElement", "[FreeCPUs > 3600]", "(UniqueID,FreeCPUs)"])
child.expect_exact("ComputingElement [FreeCPUs > 3600] has 4 -> CESEBind, Cluster or VOView")
child.expect_exact("UniqueID : cclcgceli01.in2p3.fr:2119/jobmanager-bqs-medium")
child.expect("3699")
child.expect("4378")
child.expect("3938")
child.expect("4000")
checkStatus(child, 0)

# Missing quote
child = pexpect.spawn("saga-isn", ['-n', "1", "glue1", 'Service', "[Name = glite-data-transfer-fts_fts.esac.esa.int-channel']"])
child.expect_exact("SAGA(BadParameter): Unable to parse Name")
checkStatus(child, 1)

# Bad entity set name
child = pexpect.spawn("saga-isn", ["glue1", "service"])
child.expect_exact("SAGA(BadParameter): Invalid entity name: service")
checkStatus(child, 1)

#child = pexpect.spawn("saga-isn", ["-t", "glue1", "Service", "[URI = 'lfc03.pic.es']", "UP"])
#child.logfile = sys.stdout
#checkStatus(child, 0)

# child = pexpect.spawn("saga-isn", ["-t", "glue1", "Service", "[URI = 'lfc03.pic.es']", "DOWN"])
# child.logfile = sys.stdout
# checkStatus(child, 0)



