#!/usr/bin/perl -w
#
# Copyright (c) Members of the EGEE Collaboration. 2009-2010.
# See http://www.eu-egee.org/partners/ for details on the copyright
# holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
use strict;

sub CheckInfoServiceUrl($$);
sub RestartBDII();
sub RunTests();

my $TEST_BDII=shift;
if (! $TEST_BDII) {
    print "Please specify the name of the machine set up as a test BDII\n";
    exit 1;
}

my $BDII_URL = "ldap://".$TEST_BDII.":2170";

my $BDII_WAIT=shift;
if (! $BDII_WAIT) {
    print "Please specify the number of seconds to wait for the BDII to reinitialize itself\n";
    exit 1;
}

my $cmd;

if (-e "default.ldif.gz") {
    print "Unpacking default.ldif\n";$cmd = "ssh root@".$TEST_BDII." service bdii restart";
    $cmd = "gunzip -f default.ldif.gz";
    if (system($cmd) != 0) {exit 1};
}

print "Set up fixed BDII configuration from CVS\n";
$cmd = "scp ../../org.glite.saga-adapter.sd-cpp/test/default.ldif root@".$TEST_BDII.":/opt/glite/etc/gip/ldif/default.ldif";
if (system($cmd) != 0) {exit 1};
RestartBDII();


# Set BDII_DIR env variable
$ENV{BDII_URL} = $BDII_URL;

RunTests();


sub RunTests() {
    foreach ("python functions.py") {
	print "\nNow running ", $_, "\n";
	if (system($_) != 0) {exit 1};
    }
}

sub RestartBDII() {
    my $cmd = "ssh root@".$TEST_BDII." service bdii restart";
    if (system($cmd) != 0) {exit 1};
    print "Now sleep for ", $BDII_WAIT, " seconds\n";
    sleep($BDII_WAIT);
    my $output = qx(saga-sd --af "" --sa -n 1) or die();
    print $output;
}






